//! Final output structs

use serde::{Deserialize, Serialize};
use std::cmp::Ordering;

#[derive(Serialize, Deserialize, Eq)]
pub struct User {
    pub id: u64,
    pub user_name: String,
}

/// When ordering, only the user ID is taken into account.
impl Ord for User {
    fn cmp(&self, other: &Self) -> Ordering {
        self.id.cmp(&other.id)
    }
}

impl PartialOrd for User {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        Some(self.cmp(other))
    }
}

impl PartialEq for User {
    fn eq(&self, other: &Self) -> bool {
        self.id == other.id
    }
}

#[derive(Serialize, Deserialize, Eq)]
pub struct Post {
    pub id: u64,
    pub thread: u64,
    pub content: String,
    pub page: u64,
    pub creation_time: u64,
    pub updated_time: u64,
    pub poster: User,
    pub thread_index: u64, // Post # in Thread
}

impl Ord for Post {
    fn cmp(&self, other: &Self) -> Ordering {
        self.thread_index.cmp(&other.thread_index)
    }
}

impl PartialOrd for Post {
    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
        Some(self.cmp(other))
    }
}

impl PartialEq for Post {
    fn eq(&self, other: &Self) -> bool {
        self.thread_index == other.thread_index
    }
}

#[derive(Serialize, Deserialize)]
pub struct Thread {
    pub id: u64,
    pub title: String,
    pub posts: Vec<PostDocument>,
    pub post_time: u64,
    //    pub poster_id: u64,
    pub participants: Vec<UserDocument>,
}

#[derive(Serialize, Deserialize)]
pub struct PostDocument {
    id: u64,
    thread_id: u64,
    poster_id: u64,
    poster_user_name: String,
    post_time: u64,
    content: String,
}

impl PostDocument {
    pub fn new(
        id: u64,
        thread_id: u64,
        poster_id: u64,
        poster_user_name: String,
        post_time: u64,
        content: String,
    ) -> PostDocument {
        PostDocument {
            id,
            thread_id,
            poster_id,
            poster_user_name,
            post_time,
            content,
        }
    }
}

#[derive(Serialize, Deserialize)]
pub struct ThreadDocument {
    id: u64,
    post_time: u64,
    title: String,
}

impl ThreadDocument {
    pub fn new(id: u64, post_time: u64, title: String) -> ThreadDocument {
        ThreadDocument {
            id,
            post_time,
            title,
        }
    }
}

#[derive(Serialize, Deserialize, Eq)]
pub struct UserDocument {
    pub id: u64,
    user_name: String,
}

impl UserDocument {
    pub fn new(id: u64, user_name: String) -> UserDocument {
        UserDocument { id, user_name }
    }
}

impl Ord for UserDocument {
    fn cmp(&self, other: &Self) -> std::cmp::Ordering {
        self.id.cmp(&other.id)
    }
}

impl PartialOrd for UserDocument {
    fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
        Some(self.cmp(other))
    }
}

impl PartialEq for UserDocument {
    fn eq(&self, other: &Self) -> bool {
        self.id == other.id
    }
}
